import sys
from jet_files import (
    status,
    push,
    pull,
    list_commits,
    commit_changeset,
    init,
    add,
    merge,
    help_text,
    local_tests,
    login,
    revert,
    hook,
    branch,
    diff,
    conflicts,
    setup,
    stash
)


commands = {
    "add": add.run,
    "push": push.run,
    "pull": pull.run,
    "commit": commit_changeset.run,
    "merge": merge.run,
    "status": status.run,
    "init": init.run,
    "help": help_text.run,
    "list": list_commits.run,
    "test": local_tests.run,
    "login": login.run,
    "hook": hook.run,
    "branch": branch.run,
    "revert": revert.run,
    "switch": branch.switch,
    "branches": branch.display,
    "diff": diff.run,
    "delete": branch.delete_branch,
    "resolve": conflicts.resolve,
    "conflicts": conflicts.list_conflicts,
    "setup": setup.run,
    "clone": pull.clone,
    "stash": stash.run,
    "unstash": stash.unstash,
}


def run():
    try:
        commands[sys.argv[1]]()
    except KeyError:
        print "Invalid Command - Please see www.jetvc.co.uk/documentation/ " \
            "for more info!"
    except IndexError, e:
        print "Not enough arguments"



import os
from shutil import copyfile, rmtree
import helper_functions as hf
import sys


def branch(branch_name):
    # Checking that there is no un-committed changed files
    filename = os.path.join(hf.get_branch_location() + 'changeset.txt')
    # Assume nothing has changed...
    changed_files = False
    # If there is a uncommitted changeset...
    if os.path.isfile(filename):
        changed_files = True
    # If any hashes don't match up
    if len(hf.get_changed_files(None, None)) > 0:
        changed_files = True
    # Error if there are uncommitted changed files...
    if changed_files:
        print "You can't branch until you commit...."
        return
    # END changed files check

    # Checking that the given branch name is ok
    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        if os.path.exists(os.path.join(branches_path + branch_name))\
                or branch_name == 'master':
            # Name exists, error 
            print "Already a branch with that name... please try another!"
            return
        else:
            # Make directory for the branch
            os.mkdir(os.path.join(branches_path + branch_name))
    else:
        # Make the required directories
        os.mkdir(os.path.join(hf.get_jet_directory() + '/.jet/branches/'))
        os.mkdir(os.path.join(branches_path + branch_name))
    # All ok, starting branching.

    f = []  # Contains the full path and filenames of all files to be branched
    filenames_list = []  # Contains all the file names (not full path)
    # Trawl all files in the users repo'd directory
    for (dirpath, dirnames, filenames) in os.walk(hf.get_jet_directory()):
        for filename in filenames:
            # Make sure no ignored files are present
            if hf.filter_one_file_by_ignore(filename):
                # No jet files to be included
                if not '.jet' in dirpath:
                    # Add files name to filename list
                    filenames_list.append(filename)
                    # Add full path and filename to f
                    f.append(os.path.join(dirpath, filename))

    # Create a new file listing all the current files and hashes at this point
    file_name = os.path.join(hf.get_jet_directory()
                             + '/.jet/branches/%s/latest_saved_files'
                             % branch_name)
    with open(file_name, 'w') as file_:
        for file_to_add in f:
            file_.write(file_to_add + "\n")
            file_.write(hf.checksum_md5(file_to_add) + "\n")
    # Make a folder for the init commit of the branch
    os.mkdir('.jet/branches/%s/0/' % branch_name)
    # Store the files that are in the branch at the first commit
    file_name = os.path.join(hf.get_jet_directory()
                             + '/.jet/branches/%s/0/file_log.txt'
                             % branch_name)
    with open(file_name, 'w') as file_:
        for file_to_add in f:
            file_.write(file_to_add + "\n")

    count = 0
    for file_to_add in f:
        # Make a folder to store details of that file
        folder = os.path.join(hf.get_jet_directory() +
                              '/.jet/branches/%s/%s/%s' % (branch_name,
                                                           0,
                                                           count))
        os.mkdir(folder)
        # Storing the full filename
        filename = os.path.join(hf.get_jet_directory() +
                                '/.jet/branches/%s/%s/%s/filename.txt'
                                % (branch_name, 0, count))
        with open(filename, 'w') as myFile:
                myFile.write(file_to_add)
        filename = filenames_list[count]
        # Store the contents of the file
        copyfile(file_to_add, os.path.join(hf.get_jet_directory()
                                           + '/.jet/branches/%s/0/%s/%s'
                                           % (branch_name,
                                              count,
                                              filename)))
        count += 1

    # Branch making complete, store details of branch
    print "Branch %s made" % branch_name
    # Storing parent branch
    filename = os.path.join(hf.get_jet_directory()
                            + '/.jet/branches/%s/parent' % branch_name)
    old_branch = hf.get_branch()
    old_commit = hf.get_commit()
    with open(filename, 'w') as file_:
        file_.write(old_branch)  # Old branch name
        file_.write('\n')
        file_.write(old_commit)  # Old branch commit number
                                 # from where branched off

    # Now storing which is the current branch
    filename = os.path.join(hf.get_jet_directory() + '/.jet/branch')
    with open(filename, 'w') as file_:
        file_.write(branch_name)  
    # Now storing the commit number (0)
    filename = os.path.join(hf.get_jet_directory() + '/.jet/current_commit')
    with open(filename, 'w') as file_:
        file_.write("0")
    # All done
    print "You are now working in branch %s" % branch_name


def run():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 3:
        print "Please form branch commands by typing" \
              " $jet branch <branch_name>"
        return
    branch_name = sys.argv[2]
    branch(branch_name)


def switch():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 3:
        print "Please form your switch commands $jet switch <branch_name>"
        return

    # Checks for any changed files, as you must have committed first!!!
    filename = os.path.join(hf.get_branch_location() + 'changeset.txt')
    # Assumes nothing has changed
    changed_files = False
    # If there is a changeset uncommitted
    if os.path.isfile(filename):
        changed_files = True
    # If any hashes don't match up...
    if len(hf.get_changed_files(None, None)) > 0:
        changed_files = True
    # Error if anything changed!! 
    if changed_files:
        print "You can't switch branch until you commit...."
        return

    # Master info stored separately, hence the if statement
    if sys.argv[2] == 'master':
        # Record master as the branch currently on
        filename = os.path.join(hf.get_jet_directory() + '.jet/branch')
        with open(filename, 'w') as file_:
            file_.write(sys.argv[2])
        # Do the revert process
        hf.revert(sys.argv[2], hf.get_highest_commit(sys.argv[2]))
        # Update the latest saved files with ones from new branch
        filename = os.path.join(hf.get_branch_location()
                                + 'latest_saved_files')
        # Delete previous save
        os.remove(filename)
        with open(filename, 'w') as file_:
            for file_to_add in hf.get_current_files(None):
                file_.write(file_to_add + "\n")
                file_.write(hf.checksum_md5(file_to_add) + "\n")
        # All done!
        print "Successfully switched to branch %s" % sys.argv[2]
        return
    # Only here if the branch to switch to isn't master
    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        if os.path.exists(os.path.join(branches_path + sys.argv[2])):
            filename = os.path.join(hf.get_jet_directory() + '.jet/branch')
            # Setting the new branch name as current branch
            with open(filename, 'w') as file_:
                file_.write(sys.argv[2])
            hf.revert(sys.argv[2], hf.get_highest_commit(sys.argv[2]))
            filename = os.path.join(hf.get_branch_location()
                                    + 'latest_saved_files')
            os.remove(filename)
            with open(filename, 'w') as file_:
                for file_to_add in hf.get_current_files(None):
                    file_.write(file_to_add + "\n")
                    file_.write(hf.checksum_md5(file_to_add) + "\n")
            print "Successfully switched to branch %s" % sys.argv[2]
        else:
            print "Invalid branch name"
    else:
        print "Invalid branch name"


def display():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    print "Branch name (parent)"
    print "Master(root)"
    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        branches = hf.get_immediate_subdirectories(branches_path)
        for b in branches:
            print "%s (%s)" % (b, hf.get_parent(b))

    print "You are currently on branch %s" % hf.get_branch()


def delete_branch():
    if len(sys.argv) != 3:
        print "Please form your delete commands $jet delete <branch_name>"
        return
    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        if not os.path.exists(os.path.join(branches_path + sys.argv[2])):
            print "Invalid branch name, please try again."
            return
    else:
        print "Invalid branch name, please try again"
        return
    if sys.argv[2] == 'master':
        print "Cannot delete master"
        return
    delete(sys.argv[2])


def delete(branch_name):
    directory = os.path.join(hf.get_jet_directory()
                             + '/.jet/branches/%s/' % branch_name)
    rmtree(directory)
    print "Deleted"



import os
from jet_files import helper_functions as hf
import getpass
import requests

#DOMAIN = 'http://0.0.0.0:8000/'
DOMAIN = 'http://www.jetvc.co.uk/'


def setup():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return

    status_code = 403
    first_time = True
    print "Connecting...."
    # Stop syntax highlighting, set variables
    username = None
    response = None
    while not status_code == 200:
        if not first_time:
            print hf.BColors.RED + \
                "Incorrect details - please try again!" + \
                hf.BColors.ENDC
        print "Please type your Jet username used in registration" \
              " at www.jetvc.co.uk. If you're not already " \
              "registered please visit www.jetvc.co.uk/register/"
        username = raw_input("Username: ")
        password = getpass.getpass()

        url = "%slogin_user/" % DOMAIN
        try:
            data = {
                'username': username,
                'password': password,
            }
            response = requests.post(url, data=data)
            first_time = False
            status_code = response.status_code
        except Exception, e:
            print "Failed to connect to Jets servers."
            print "Error - %s" % e
            return
    print "Congrats you are logged in. These details will be saved"
    filename = os.path.join(hf.get_jet_directory()
                            + '/.jet/username')
    with open(filename, 'w') as file_:
        file_.write(username + '\n')
        file_.write(response.headers['user_id'])
    print "Re-run setup to change user"

    user_id = response.headers['user_id']

    status_code = 400
    first_time = True
    while not status_code == 200:
        if not first_time:
            print hf.BColors.RED + \
                "Repository name not found - please try again!" + \
                hf.BColors.ENDC
        print "Please type the name of the online repo you wish" \
              " to link this with."
        repo_name = raw_input("Name (case-sensitive): ")
        try:
            url = "%sverify_repo/" % DOMAIN
            data = {
                'repo_name': repo_name,
                'user_id': user_id,
            }
            response = requests.post(url, data=data)
            first_time = False
            status_code = response.status_code
        except Exception, e:
            print "Failed to connect to Jets servers."
            print "Error - %s" % e
            return
    print "All setup."
    print "These details will be saved"
    filename = os.path.join(hf.get_jet_directory()
                            + '/.jet/repo_id')
    with open(filename, 'w') as file_:
        file_.write(response.headers['repo_id'])


def run():
    setup()


import os
from jet_files import helper_functions as hf


def status():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    cwd = os.getcwd()
    branch = hf.get_branch()
    branch_location = hf.get_branch_location_param(branch)
    jet_directory = hf.get_jet_directory()
    print "You are working on branch %s" % branch
    current_files = hf.get_current_files(jet_directory)
    stored_files_and_hashes = hf.get_stored_files_and_hashes()
    stored_files = hf.get_stored_files(stored_files_and_hashes)
    changed_files = hf.get_changed_files(current_files,
                                         stored_files_and_hashes)
    if len(current_files) == len(stored_files)\
            and len(changed_files) == 0:
        print "Nothing has changed!"
    else:
        print "Your repo has changed since the last commit"
        files_in_changeset = hf.get_files_in_changeset(branch_location)
        new_files = hf.get_new_files(current_files, stored_files)
        if new_files:
            new_files_in_changeset =\
                hf.get_new_files_in_changeset(files_in_changeset)
            if new_files_in_changeset:
                unadded_new = []
                for new_file in new_files:
                    if new_file not in new_files_in_changeset:
                        unadded_new.append(new_file)
                print "New files in changeset:"
                for new_file_in_changeset in new_files_in_changeset:
                    print hf.BColors.GREEN + \
                        "    %s" % hf.relative(new_file_in_changeset, cwd) +\
                        hf.BColors.ENDC
            else:
                unadded_new = new_files
            if unadded_new:
                print "New files:"
                for new_file in unadded_new:
                    print hf.BColors.GREEN + \
                        "    %s" % hf.relative(new_file, cwd) + \
                        hf.BColors.ENDC
        deleted_files = hf.get_deleted_files(current_files, stored_files)
        if deleted_files:
            deleted_files_in_changeset =\
                hf.get_deleted_files_in_changeset(files_in_changeset)
            if deleted_files_in_changeset:
                unadded_deleted = []
                for deleted_file in deleted_files:
                    if deleted_file not in deleted_files_in_changeset:
                        unadded_deleted.append(deleted_file)
                print "Deleted files in changeset:"
                for deleted_file_in_changeset in deleted_files_in_changeset:
                    print hf.BColors.RED + \
                        "    %s" % hf.relative(deleted_file_in_changeset, cwd)\
                        + hf.BColors.ENDC
            else:
                unadded_deleted = deleted_files
            if unadded_deleted:
                print "Deleted files:"
                for deleted_file in unadded_deleted:
                    print hf.BColors.RED + \
                        "    %s" % hf.relative(deleted_file, cwd) +\
                        hf.BColors.ENDC

        if changed_files:
            changed_files_in_changeset =\
                hf.get_changed_files_in_changeset(files_in_changeset)
            if changed_files_in_changeset:
                unadded = []
                for file_ in changed_files:
                    if file_ not in changed_files_in_changeset:
                        unadded.append(file_)
                print "Changed files in changeset:"
                for changed_file_in_changeset in changed_files_in_changeset:
                    print hf.BColors.YELLOW + \
                        "    %s" % hf.relative(changed_file_in_changeset, cwd)\
                        + hf.BColors.ENDC
            else:
                unadded = changed_files
            if unadded:
                print "Changed files:"
                for changed_file in unadded:
                    print hf.BColors.YELLOW +\
                        "    %s" % hf.relative(changed_file, cwd) +\
                        hf.BColors.ENDC


def run():
    status()



import os
from os import walk
import hashlib
import subprocess


class BColors:
    def __init__(self):
        pass

    PINK = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    ENDC = '\033[0m'


def get_jet_directory():
    directory = os.getcwd()
    parent = os.path.abspath(os.path.join(directory, os.pardir))
    jet_directory = ""
    found = False
    while not directory == parent and jet_directory == "":
        for filename in os.listdir(directory):
            if filename == ".jet":
                found = True
        if found:
            jet_directory = directory
        else:
            directory = parent
            parent = os.path.abspath(os.path.join(directory, os.pardir))
    return jet_directory


def get_branch_location():
    branch = get_branch()
    if branch == 'master':
        return os.path.join(get_jet_directory() + '/.jet/')
    else:
        return os.path.join(get_jet_directory() + '/.jet/branches/%s/'
                            % branch)


def get_branch_location_param(branch):
    if branch == 'master':
        return os.path.join(get_jet_directory() + '/.jet/')
    else:
        return os.path.join(get_jet_directory() + '/.jet/branches/%s/'
                            % branch)


def get_immediate_subdirectories(directory):
    return [name for name in os.listdir(directory)
            if os.path.isdir(os.path.join(directory, name))]


def get_current_files(jet_directory):
    if jet_directory is None:
        jet_directory = get_jet_directory()
    file_list = []
    for (dirpath, dirnames, filenames) in walk(jet_directory):
        if '.jet' in dirpath:
            continue
        for filename in filenames:
            file_list.append(os.path.join(dirpath, filename))

    return filter_files_by_ignore(file_list)


def get_new_commit_number():
    commits = get_immediate_subdirectories(get_branch_location())
    try:
        commits.remove('branches')
    except ValueError:
        pass
    biggest = 0
    for commit in commits:
        try:
            commit_num = int(commit)
            if commit_num > biggest:
                biggest = commit_num
        except ValueError:
            pass
    int_latest = int(biggest)
    new_commit_number = int_latest + 1
    return new_commit_number


def get_stored_files_and_hashes():
    # TODO - change this to split by lines
    filename = os.path.join(get_branch_location() + 'latest_saved_files')
    with open(filename, 'r') as myFile:
        data = myFile.read().splitlines()
    return data
    # word = []
    # lines = []
    # code = []
    # for char in data:
    #     if char == "~" and len(code) == 0:
    #         code.append(char)
    #     elif char == "J" and len(code) == 1:
    #         code.append(char)
    #     elif char == "/" and len(code) == 2:
    #         code.append(char)
    #     elif char == "E" and len(code) == 3:
    #         code.append(char)
    #     elif char == "T" and len(code) == 4:
    #         code.append(char)
    #         new_line = ''.join(word)
    #         lines.append(new_line)
    #         word = []
    #         code = []
    #     else:
    #         word.extend(code)
    #         if not char == "~":
    #             code = []
    #             word.append(char)
    # if len(word) > 0:
    #     lines.append(''.join(word[:-5]))
    # return lines


def get_stored_files(lines):
    if lines is None:
        lines = get_stored_files_and_hashes()
    return lines[::2]


def get_stored_hash(filename, stored_files_and_hashes):
    if not stored_files_and_hashes:
        stored_files_and_hashes = get_stored_files_and_hashes()
    return_next = False
    for line in stored_files_and_hashes:
        if return_next:
            return line
        if line == filename:
            return_next = True
    return False


def get_new_files(current_files, stored_files):
    if current_files is None:
        current_files = get_current_files(None)
    if stored_files is None:
        stored_files = get_stored_files(None)
    return [x for x in current_files if x not in stored_files]


def get_files_in_changeset(branch_location):
    if branch_location is None:
        branch_location = get_branch_location()
    try:
        filename = os.path.join(branch_location + 'changeset.txt')
        with open(filename, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        return []
    return lines


def get_new_files_in_changeset(lines):
    if lines is None:
        try:
            filename = os.path.join(get_branch_location() + 'changeset.txt')
            with open(filename, 'r') as myFile:
                lines = myFile.read().splitlines()
        except IOError:
            return []
    new_files = []
    for line in lines:
        if line.startswith('+'):
            new_files.append(line[1:])
    return new_files


def get_deleted_files_in_changeset(lines):
    if lines is None:
        try:
            filename = os.path.join(get_branch_location() + 'changeset.txt')
            with open(filename, 'r') as myFile:
                lines = myFile.read().splitlines()
        except IOError:
            return []
    deleted_files = []
    for line in lines:
        if line.startswith('-'):
            deleted_files.append(line[1:])
    return deleted_files


def get_changed_files_in_changeset(lines):
    if lines is None:
        try:
            filename = os.path.join(get_branch_location() + 'changeset.txt')
            with open(filename, 'r') as myFile:
                lines = myFile.read().splitlines()
        except IOError:
            return []
    changed_files = []
    for line in lines:
        if line.startswith('~'):
            changed_files.append(line[1:])
    return changed_files


def get_deleted_files(current_files, stored_files):
    if current_files is None:
        current_files = get_current_files(None)
    if stored_files is None:
        stored_files = get_stored_files(None)
    deleted_files = []
    for stored_file in stored_files:
        if stored_file not in current_files:
            deleted_files.append(stored_file)
    return deleted_files


def already_initialized():
    return not get_jet_directory() == ""


def checksum_md5(filename):
    with open(filename, 'r') as f:
        contents = f.read()
    return hashlib.md5(contents).hexdigest()


def old_checksum_md5(filename):
    md5 = hashlib.md5()
    with open(filename, 'rb') as f:
        for chunk in iter(lambda: f.read(8192), b''): 
            md5.update(chunk)
    return md5.digest()


def get_changed_files(current_files, stored_files_and_hashes):
    if current_files is None:
        current_files = get_current_files(None)
    changed_files = []
    for file_to_compare in current_files:
        if not checksum_md5(file_to_compare) ==\
                get_stored_hash(file_to_compare, stored_files_and_hashes):
            changed_files.append(file_to_compare)
    return changed_files


def get_change_description(filename):
    commit_number = get_new_commit_number() - 2
    previous_file = get_file_at(get_branch(), commit_number, filename)
    if previous_file is None:
        return None
    difference = diff(previous_file, filename)
    if not difference:
        return "Jet is sorry, but there was an error in processing the" \
               " changes for this file"
    else:
        return difference


def diff(file1, file2):
    try:
        if type(file1) == list:
            old_lines = file1
        else:
            with open(file1, 'r') as file_:
                old_lines = file_.read().splitlines()
        if type(file2) == list:
            current_lines = file2
        else:
            with open(file2, 'r') as file_:
                current_lines = file_.read().splitlines()
    except IOError:
        return "Jet is sorry, but there was an error in processing the" \
               " changes for this file"
    description = ""
    line_number = -1
    count = -1
    old_count = -1
    while old_count < len(old_lines):
        old_count += 1
        count += 1
        line_number += 1
        try:
            line = old_lines[old_count]
        except IndexError:
            break
        try:
            current_lines[count]
        except IndexError:
            description += ("(" + str(line_number) + ") " +
                            "- " +
                            line +
                            "\n")
            continue

        if not current_lines[count] == line:
            if line == "":
                description += ("(" + str(line_number) + ") " +
                                "- blank line\n")
                count -= 1
            else:
                if current_lines[count] == "":
                    description += ("(" + str(line_number) + ") " +
                                    "+ blank line\n")
                    old_count -= 1
                else:
                    description += ("(" + str(line_number) + ") " +
                                    "~ " + current_lines[count] + "\n")

    while count <= len(current_lines) - 1:
        if current_lines[count] == "":
            description += ("(" + str(line_number) + ") " +
                            "+ blank line\n")
        else:
            description += ("(" + str(line_number) + ") " +
                            "+ " +
                            current_lines[count] +
                            "\n")
        line_number += 1
        count += 1

    description_to_return = description[:-1]
    if description_to_return == "":
        return "No changes found"
    else:
        return description_to_return


def get_file_change_number(branch, commit_number, filename):
    if not branch == 'master':
        file_list_file = os.path.join(get_jet_directory() +
                                      '/.jet/branches/%s/%s/file_log.txt'
                                      % (branch, commit_number))
    else:
        file_list_file = os.path.join(get_jet_directory() +
                                      '/.jet/%s/file_log.txt' % commit_number)
    with open(file_list_file, 'r') as myFile:
        file_list = myFile.read().splitlines()
    change_number = 0
    for file_ in file_list:
        if file_ == filename or file_[1:] == filename:
            return change_number
        else:
            change_number += 1
    return None


def get_last_complete_file(branch, filename):
    change_number = None
    count = 0
    while change_number is None and count <= get_highest_commit(branch):
        change_number = get_file_change_number(branch, count, filename)
        count += 1
    count -= 1
    if count > 0:
        name_of_file = 'changes.txt'
    else:
        name_of_file = os.path.basename(filename)
    if not branch == 'master':
        modded_filename = os.path.join(get_jet_directory()
                                       + '/.jet/branches/%s/%s/%s/%s'
                                       % (branch, count,
                                          change_number, name_of_file))
    else:
        modded_filename = os.path.join(get_jet_directory() + '/.jet/%s/%s/%s'
                                       % (count, change_number, name_of_file))
    try:
        with open(modded_filename, 'r') as myFile:
                current_file = myFile.read().splitlines()
    except IOError:
        return None, None
    commit_number = 0
    return current_file, commit_number


def get_diff_at(branch, commit_number, filename):
    change_num = get_file_change_number(branch, commit_number, filename)
    if change_num is None:
        return []
    if not branch == 'master':
        modded_filename = os.path.join(get_jet_directory()
                                       + '/.jet/branches/%s/%s/%s/changes.txt'
                                       % (branch, commit_number, change_num))
    else:
        modded_filename = os.path.join(get_jet_directory()
                                       + '/.jet/%s/%s/changes.txt'
                                       % (commit_number, change_num))
    with open(modded_filename, 'r') as myFile:
        difference = myFile.read().splitlines()
    return difference


def get_file_at(branch, commit_number, filename):
    if not is_valid_commit_number(commit_number, branch):
        return None
    last_complete, last_full_commit = get_last_complete_file(branch, filename)
    if last_complete is None:
        return None
    commits_to_add = []
    commit = last_full_commit + 1
    if commit_number == '0' or commit_number == 0:
        return last_complete
    while not int(commit) == int(commit_number):
        commits_to_add.append(commit)
        commit += 1
    commits_to_add.append(commit)

    current_file = last_complete
    for c in commits_to_add:
        current_file = reform_file(current_file, get_diff_at(branch,
                                                             c,
                                                             filename))

    return current_file


def reform_file(_file_, diff_list):
    if type(_file_) == list:
        lines = _file_
    else:
        with open(_file_, 'r') as file_:
            lines = file_.read().splitlines()
    if len(diff_list) == 0:
        return lines
    if diff_list[0] == 'No changes found':
        return lines
    for d in diff_list:
        try:
            index = d.index(' ')
            line_number_list = d[1:index-1]
            line_number = ''
            for number in line_number_list:
                line_number += number
            count = int(line_number)
        except ValueError:
            if type(_file_) == list:
                lines = _file_
            else:
                with open(_file_, 'r') as file_:
                    lines = file_.read().splitlines()
            return lines
        action = d[index + 1]
        content = d[index + 3:]
        if action == '~':
            try:
                lines[count] = content
            except IndexError:
                pass
        elif action == '+':
            try:
                if content == 'blank line':
                    lines.insert(count, '')
                else:
                    lines.insert(count, content)
            except IndexError:
                pass
        elif action == '-':
            try:
                lines[count] = "~J/E\T DELETE ~J/E\T"
            except IndexError:
                pass

    to_return = []
    for line in lines:
        if not line == "~J/E\T DELETE ~J/E\T":
            to_return.append(line)
    return to_return


def get_username():
    filename = os.path.join(get_jet_directory() + '/.jet/username')
    try:
        with open(filename, 'r') as file_:
                    lines = file_.read().splitlines()
        username = lines[0]
    except IOError:
        username = ''
    except IndexError:
        os.remove(filename)
        username = ''
    return username


def logged_in():
    return not get_username() == ''


def get_commit_hook():
    try:
        filename = (get_branch_location() + 'hooks')
        with open(filename, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        return False
    try:
        if lines[0] == 'commit':
            return lines[1]
        elif lines[2] == 'commit':
            return lines[3]
        else:
            return False
    except IndexError:
        return False


def get_push_hook():
    try:
        filename = (get_branch_location() + 'hooks')
        with open(filename, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        return False
    try:
        if lines[0] == 'push':
            return lines[1]
        elif lines[2] == 'push':
            return lines[3]
        else:
            return False
    except IndexError:
        return False


def run_hook(filename):
    # noinspection PyBroadException
    try:
        return_code = subprocess.call("python %s" % filename, shell=True)
    except Exception:
        return False
    if return_code == 0:
        return True
    else:
        return False


def is_valid_commit_number(number, branch):
    number = '%s' % number
    if branch is None:
        commits = get_immediate_subdirectories(get_branch_location())
    elif branch == 'master':
        commits = get_immediate_subdirectories(
            os.path.join(get_jet_directory() + '/.jet/'))
    else:
        commits = get_immediate_subdirectories(
            os.path.join(get_jet_directory() + '/.jet/branches/%s' % branch))
    try:
        commits.remove('branches')
    except ValueError:
        pass
    if number in commits:
        return True
    else:
        return False


def edit_commit_list(branch, commit_number, current_list):
    if branch == 'master':
        filename = os.path.join(get_jet_directory()
                                + '/.jet/%s/file_log.txt'
                                % commit_number)
    else:
        filename = os.path.join(get_jet_directory()
                                + '/.jet/branches/%s/%s/file_log.txt'
                                % (branch, commit_number))
    if commit_number == 0:
        raise AttributeError
    with open(filename, 'r') as myFile:
        edits = myFile.read().splitlines()
    for edit in edits:
        if edit.startswith('+'):
            current_list.append(edit[1:])
        elif edit.startswith('-'):
            try:
                current_list.remove(edit[1:])
            except ValueError:
                pass
        else:
            pass
    return current_list


def get_highest_commit(branch):
    if branch != 'master':
        directories = get_immediate_subdirectories(os.path.join(
            get_jet_directory() + '/.jet/branches/%s' % branch))
    else:
        directories = get_immediate_subdirectories(
            os.path.join(get_jet_directory() + '/.jet/'))
        try:
            directories.remove("branches")
        except ValueError:
            pass
    highest = 0
    for directory in directories:
        if directory > highest:
            highest = directory
    return "%s" % highest


def get_parent(branch):
    if branch == 'master':
        raise AttributeError
    filename = os.path.join(get_jet_directory()
                            + '/.jet/branches/%s/parent' % branch)
    with open(filename, 'r') as myFile:
        lines = myFile.read().splitlines()
    return lines[0]


def get_parent_commit(branch):
    if branch == 'master':
        raise AttributeError
    filename = os.path.join(get_jet_directory()
                            + '/.jet/branches/%s/parent' % branch)
    with open(filename, 'r') as myFile:
        lines = myFile.read().splitlines()
    return lines[1]


def get_file_list_at(branch, commit_number):
    if branch == 'master':
        filename = os.path.join(get_jet_directory() + '/.jet/0/file_log.txt')
    else:
        filename = os.path.join(get_jet_directory()
                                + '/.jet/branches/%s/0/file_log.txt' % branch)
    with open(filename, 'r') as myFile_:
        branch_file_list = myFile_.read().splitlines()

    if commit_number > 0:
        commit_int = int(commit_number)
        for i in range(1, commit_int + 1):
            branch_file_list = edit_commit_list(branch, i, branch_file_list)
    return branch_file_list


def revert(branch, commit_number):
    current_files = get_current_files(None)
    files_at_revert_point = get_file_list_at(branch, commit_number)
    files_to_delete = [x for x in current_files
                       if not x in files_at_revert_point]
    for file_ in files_to_delete:
        os.remove(file_)
    for file_ in files_at_revert_point:
        new_contents = get_file_at(branch, commit_number, file_)
        with open(file_, 'w') as myFile:
            for content in new_contents:
                myFile.write("%s\n" % content)

    filename = os.path.join(get_jet_directory() + '/.jet/branch')
    with open(filename, 'w') as file_:
        file_.write(branch)
    filename = os.path.join(get_jet_directory() + '/.jet/current_commit')
    with open(filename, 'w') as file_:
        file_.write(str(commit_number))


def get_branch():
    filename = os.path.join(get_jet_directory() + '/.jet/branch')
    with open(filename, 'r') as myFile:
        lines = myFile.read().splitlines()
    return lines[0]


def get_commit():
    filename = os.path.join(get_jet_directory() + '/.jet/current_commit')
    with open(filename, 'r') as myFile:
        lines = myFile.read().splitlines()
    return lines[0]


def get_joint_parent(branch_1, branch_2):
    mutual_branch = 'master'
    b1_branch_list = []
    while not branch_1 == 'master':
        b1_branch_list.append(branch_1)
        branch_1 = get_parent(branch_1)
    b2_branch_list = []
    while not branch_2 == 'master':
        b2_branch_list.append(branch_2)
        branch_2 = get_parent(branch_2)
    b1_branch_list.append('master')
    b2_branch_list.append('master')
    found = False
    for b1 in b1_branch_list:
        if found:
            continue
        for b2 in b2_branch_list:
            if b1 == b2:
                mutual_branch = b2
                found = True
                continue

    index = b2_branch_list.index(mutual_branch)
    index_of_child = index - 1
    try:
        mutual_commit = get_parent_commit(b2_branch_list[index_of_child])
    except IndexError:
        mutual_commit = 0

    return mutual_branch, mutual_commit


def merge(branch_to_merge):
    current_branch = get_branch()
    current_files = get_current_files(None)
    other_files = get_file_list_at(branch_to_merge,
                                   get_highest_commit(branch_to_merge))

    joint_parent_branch,\
        joint_parent_commit_number = get_joint_parent(current_branch,
                                                      branch_to_merge)
    parent_files = get_file_list_at(joint_parent_branch,
                                    joint_parent_commit_number)

    files_to_merge = []
    files_to_ask_about = []

    for file_ in current_files:
        if file_ in other_files:
            files_to_merge.append(file_)
        else:
            files_to_ask_about.append(file_)
    files_to_ask_about += [x for x in other_files if x not in current_files]

    for f in files_to_ask_about:
        answer = ask(f)
        if not answer:
            os.remove(f)
        else:
            if f not in current_files:
                file_contents = get_file_at(branch_to_merge,
                                            get_highest_commit
                                            (branch_to_merge),
                                            f)
                if file_contents:
                    with open(f, 'w') as myFile:
                        for line in file_contents:
                            myFile.write(line)

    for f in files_to_merge:
        if f in parent_files:
            parent_file = get_file_at(joint_parent_branch,
                                      joint_parent_commit_number,
                                      f)
        else:
            parent_file = []

        file1 = get_file_at(current_branch,
                            get_highest_commit(current_branch),
                            f)
        file2 = get_file_at(branch_to_merge,
                            get_highest_commit(branch_to_merge),
                            f)
        merge_files(f, parent_file, file1, file2)


def ask(filename):
    response = raw_input("Would you like to keep the file: %s? (yes/no) "
                         % filename)

    if response == "yes" or response == "y" or response == "Yes":
        return True
    else:
        return False


def optimize_conflicts(_file_):
    optimized = False
    start = '@@@@@@@@@@HEAD@@@@@@@@@@'
    separator = '@@@@@@@@@@SEPARATOR@@@@@@@@@@'
    end = '@@@@@@@@@@END@@@@@@@@@@'
    optimized_file = _file_
    for i in range(0, len(_file_)):
        try:
            if _file_[i] == end:
                if _file_[i+1] == start:
                    optimized = True
                    end_of_set = None
                    for j in range(i+1, len(_file_)):
                        if _file_[j] == end:
                            end_of_set = j
                            break
                    start_of_set = None
                    for k in range(i-1, -1, -1):
                        if _file_[k] == start:
                            start_of_set = k
                            break
                    if start_of_set is None and end_of_set is None:
                        continue
                    first_contents = []
                    second_contents = []
                    for y in range(start_of_set + 1, i):
                        if _file_[y] == separator:
                            for z in range(y+1, i):
                                second_contents.append(_file_[z])
                            break
                        first_contents.append(_file_[y])
                    for y in range(i+2, end_of_set):
                        if _file_[y] == separator:
                            for z in range(y+1, end_of_set):
                                second_contents.append(_file_[z])
                            break
                        first_contents.append(_file_[y])
                    optimized_file = _file_[:start_of_set] +\
                        [start] +\
                        first_contents +\
                        [separator] +\
                        second_contents +\
                        [end] +\
                        _file_[end_of_set+1:]
                    break
        except IndexError:
            continue
    if optimized:
        optimized_file = optimize_conflicts(optimized_file)
    return optimized_file


def fix_file(filename, parent, file1, file2, test=False):
    _file_ = []
    conflict = False
    parent_pointer = 0
    file1_pointer = 0
    file2_pointer = 0
    total_length = max(len(parent), len(file1), len(file2))
    for i in range(len(parent), total_length):
        parent.append("")
    for i in range(len(file1), total_length):
        file1.append("")
    for i in range(len(file2), total_length):
        file2.append("")
    while parent_pointer < total_length:
        # if two versions are the same
        if file2[file2_pointer] == file1[file1_pointer]:
            # append the line to the final file
            _file_.append(file1[file1_pointer])
            # increment all pointers
            parent_pointer += 1
            file1_pointer += 1
            file2_pointer += 1
        # if file2 was changed, but not file1
        elif file1[file1_pointer] == parent[parent_pointer] and \
                not file2[file2_pointer] == parent[parent_pointer]:
            # append the line to the final file
            _file_.append(file2[file2_pointer])
            # increment all pointers - SHOULD I?!?!?
            parent_pointer += 1
            file1_pointer += 1
            file2_pointer += 1
        # if file1 was changed, but not file2
        elif file2[file2_pointer] == parent[parent_pointer] and \
                not file1[file1_pointer] == parent[parent_pointer]:
            # append the line to the final file
            _file_.append(file1[file1_pointer])
            # increment all pointers - SHOULD I?!?!?
            parent_pointer += 1
            file1_pointer += 1
            file2_pointer += 1
        else:
            _file_.append('@@@@@@@@@@HEAD@@@@@@@@@@')
            _file_.append(file1[file1_pointer])
            _file_.append('@@@@@@@@@@SEPARATOR@@@@@@@@@@')
            _file_.append(file2[file2_pointer])
            _file_.append('@@@@@@@@@@END@@@@@@@@@@')

            conflict = True
            parent_pointer += 1
            file1_pointer += 1
            file2_pointer += 1
    if conflict and not test:
        add_conflict(filename)

    count = 0
    for f in reversed(_file_):
        if not f == '':
            break
        else:
            count += 1
    if count > 0:
        del _file_[-count:]
    return optimize_conflicts(_file_)


def merge_files(filename, parent, file1, file2):
    # base case
    if file1 == file2:
        new_file = file1
    else:
        # merging is required
        try:
            new_file = fix_file(filename, parent, file1, file2)
        except IndexError:
            # error has happened. Apply worst case scenario.
            new_file = \
                ['@@@@@@@@@@HEAD@@@@@@@@@@\n'] \
                + file1 \
                + ['\n@@@@@@@@@@SEPARATOR@@@@@@@@@@\n'] \
                + file2 \
                + ['\n@@@@@@@@@@END@@@@@@@@@@']
            add_conflict(filename)

    with open(filename, 'w') as myFile:
        for line in new_file:
            myFile.write('%s\n' % line)


def is_conflicts():
    return not len(get_conflicts()) == 0


def get_conflicts():
    file_ = os.path.join(get_branch_location() + 'conflicts')
    try:
        with open(file_, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        lines = []
    return lines


def add_conflict(filename):
    conflicts = get_conflicts()
    if filename not in conflicts:
        conflicts.append(filename)

    file_ = os.path.join(get_branch_location() + 'conflicts')

    print BColors.RED + "Merge conflict for file %s" % filename + BColors.ENDC

    with open(file_, 'w') as myFile:
        for line in conflicts:
            myFile.write('%s\n' % line)


def resolve_conflict(filename):
    conflicts = get_conflicts()
    if filename in conflicts:
        conflicts.remove(filename)
    else:
        return -1

    file_ = os.path.join(get_branch_location() + 'conflicts')

    with open(file_, 'w') as myFile:
        for line in conflicts:
            myFile.write('%s\n' % line)

    return len(conflicts)


def filter_one_file_by_ignore(filename):
    try:
        name_of_jet_ignore = os.path.join(get_jet_directory() + '/.jet_ignore')
        with open(name_of_jet_ignore, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        return filter_file_by_ignore(filename, [])
    return filter_file_by_ignore(filename, lines)


# True for passed, false for ignore
def filter_file_by_ignore(filename, lines):
    for line in lines:
        if line.startswith('*') and line.endswith('*'):
            if filename.contains(line[1:-1]):
                return False
        if line.startswith('*'):
            if filename.endswith(line[1:]):
                return False
        if line.endswith('*'):
            if filename.startswith(line[:1]):
                return False
        if line == filename:
            return False
        cwd = os.getcwd()
        relative_filename = filename[len(cwd):]
        if relative_filename.startswith('/'):
            relative_filename = relative_filename[1:]
        if relative_filename.startswith(line):
            return False
    if filename.endswith("~"):
        return False

    # if none of it matches
    return True


def filter_files_by_ignore(filenames):
    try:
        name_of_jet_ignore = os.path.join(get_jet_directory() + '/.jet_ignore')
        with open(name_of_jet_ignore, 'r') as myFile:
            lines = myFile.read().splitlines()
    except IOError:
        return [x for x in filenames if (filter_file_by_ignore(x, []))]
    return [x for x in filenames if (filter_file_by_ignore(x, lines))]


def relative(filename, cwd):
    if cwd in filename:
        to_return = filename[len(cwd):]
        if to_return.startswith('/'):
            return to_return[1:]
        else:
            return to_return
    else:
        h, t = os.path.split(filename)
        if h in cwd:
            return '../%s' % t
        head = filename
        to_append = []
        count = 0
        while head not in cwd:
            head, tail = os.path.split(head)
            count += 1
            to_append.append(tail)
        back_slashes = []
        count -= 1
        while count > 0:
            back_slashes.append('../')
            count -= 1
        to_append.reverse()
        relative_name = "%s%s" % (''.join(back_slashes),
                                  '/'.join(to_append))
        return relative_name


def get_user_id():
    filename = os.path.join(get_jet_directory() + '/.jet/username')
    try:
        with open(filename, 'r') as file_:
                    lines = file_.read().splitlines()
        user_id = lines[1]
    except IOError:
        user_id = None
    except IndexError:
        user_id = None
    return user_id


def get_repo_id():
    filename = os.path.join(get_jet_directory() + '/.jet/repo_id')
    try:
        with open(filename, 'r') as file_:
            lines = file_.read().splitlines()
        repo_id = lines[0]
    except IOError:
        repo_id = None
    return repo_id


def is_setup():
    return get_user_id() and get_repo_id()


def make_directories(filename, clone):
    if clone:
        jet_directory = os.getcwd()
    else:
        jet_directory = get_jet_directory()
    stripped_filename = filename[len(jet_directory):]
    if stripped_filename.startswith('/'):
        stripped_filename = stripped_filename[1:]
    fname = os.path.basename(stripped_filename)
    folders = stripped_filename[:-len(fname)]
    try:
        os.makedirs(folders)
    except OSError:
        pass


def get_last_server_pull(branch):
    filename = os.path.join(get_branch_location_param(branch) + 'last_pull')
    try:
        with open(filename, 'r') as file_:
            lines = file_.read().splitlines()
        last_pull = int(lines[0])
    except IOError:
        last_pull = -1
    return last_pull


def save_last_pull(branch, new_number):
    filename = os.path.join(get_branch_location_param(branch) + 'last_pull')
    with open(filename, 'w') as file_:
        file_.write(str(new_number))


def get_last_update(branch):
    filename = os.path.join(get_branch_location_param(branch) + 'last_update')
    try:
        with open(filename, 'r') as file_:
            lines = file_.read().splitlines()
        commit_number = int(lines[0])
    except IOError:
        commit_number = 0
    return branch, commit_number


def add_update(branch, commit):
    filename = os.path.join(get_branch_location_param(branch) + 'last_update')
    with open(filename, 'w') as file_:
        file_.write(str(commit))


import sys
import os
import helper_functions as hf


def login():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 3:
        print "To login, type: \n    $jet login <username> \nThis could" \
              " be the username you use on www.jetvc.co.uk or one you" \
              " wish to put with your commits"
    else:
        username = sys.argv[2]
        filename = os.path.join(hf.get_jet_directory()
                                + '/.jet/username')
        with open(filename, 'w') as file_:
            file_.write(username)
        print hf.BColors.GREEN + "Welcome %s" % username + hf.BColors.ENDC


def run():
    login()


import os
import helper_functions as hf
import sys


def hook():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 4:
        print "Sorry, but that is not a recognized Jet command. Please" \
              " either type '$ jet hook <commit|push> inspect' to see " \
              "what hooks you already have, or type '$ jet hook <commit|pu" \
              "sh> <script_name>' " \
              "to add a hook!"
        return
    if sys.argv[3] == 'inspect':
        filename = (hf.get_branch_location() + 'hooks')
        if sys.argv[2] == 'commit' or sys.argv[2] == 'push':
            try:
                with open(filename, 'r') as file_:
                    lines = file_.read().splitlines()
            except IOError:
                print "You have no hooks on the %s command" % sys.argv[2]
                return
            try:
                if lines[0] == sys.argv[2]:
                    print "The %s command is hooked by:" % sys.argv[2]
                    print "    %s" % lines[1]
            except IndexError:
                print "You have no hooks on the %s command" % sys.argv[2]
            try:
                if lines[2] == sys.argv[2]:
                    print "The %s command is hooked by:" % sys.argv[2]
                    print "    %s" % lines[3]
            except IndexError:
                print "You have no hooks on the %s command" % sys.argv[2]
        else:
            print "Command isn't recognized by Jet, please form inspect" \
                  "commands like '$ jet hook <commit|push> inspect'"
    elif sys.argv[3] == 'remove':
        if sys.argv[2] == 'commit' or sys.argv[2] == 'push':
            filename = (hf.get_branch_location() + 'hooks')
            to_keep = []
            try:
                with open(filename, 'r') as file_:
                    lines = file_.read().splitlines()
                if not lines[0] == sys.argv[2]:
                    to_keep.append(lines[0])
                    to_keep.append(lines[1])
                    print "Successfully removed the %s hook" % sys.argv[2]
                if not lines[2] == sys.argv[2]:
                    to_keep.append(lines[2])
                    to_keep.append(lines[3])
                    print "Successfully removed the %s hook" % sys.argv[2]
            except IOError:
                to_keep.append("")
                to_keep.append("")
            except IndexError:
                to_keep.append("")
                to_keep.append("")

            with open(filename, 'w') as myFile:
                myFile.write(to_keep[0] + '\n')
                myFile.write(to_keep[1] + '\n')

        else:
            print "Command isn't recognized by Jet, please form inspect" \
                  "commands like '$ jet hook <commit|push> remove'"
    else:
        if not os.path.isfile(sys.argv[3]):
            print "Couldn't add hook, unrecognized file!"
        else:
            if sys.argv[2] == 'commit' or sys.argv[2] == 'push':
                filename = (hf.get_branch_location() + 'hooks')
                to_keep = []
                try:
                    with open(filename, 'r') as file_:
                        lines = file_.read().splitlines()
                    if not lines[0] == sys.argv[2]:
                        to_keep.append(lines[0])
                        to_keep.append(lines[1])
                        print "Successfully added the %s hook" % sys.argv[2]
                    if not lines[2] == sys.argv[2]:
                        to_keep.append(lines[2])
                        to_keep.append(lines[3])
                        print "Successfully added the %s hook" % sys.argv[2]
                except IOError:
                    to_keep.append("")
                    to_keep.append("")
                    print "Successfully added the %s hook" % sys.argv[2]
                except IndexError:
                    to_keep.append("")
                    to_keep.append("")

                with open(filename, 'w') as myFile:
                    myFile.write(sys.argv[2] + '\n')
                    myFile.write(sys.argv[3] + '\n')
                    myFile.write(to_keep[0] + '\n')
                    myFile.write(to_keep[1] + '\n')

            else:
                print "Command isn't recognized by Jet, please form " \
                      "commands like '$ jet hook <commit|push> <script_name>'"


def run():
    hook()


import os
from jet_files import helper_functions as hf
import sys


def revert():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 4:
        print "Please form revert commands by typing" \
              " '$jet revert <branch_name> <commit_number>' \n" \
              "Remember the default branch is called 'master'"
        return

    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        if not sys.argv[2] == 'master':
            if not os.path.exists(os.path.join(branches_path + sys.argv[2])):
                print "Invalid branch name, aborting revert"
                print "Please form revert commands by typing" \
                      " '$jet revert <branch_name> <commit_number>' \n" \
                      "Remember the default branch is called 'master'"
                return

    if hf.is_valid_commit_number(sys.argv[3], sys.argv[2]):
        response = raw_input("Revert changes all your files back to the "
                             "point they were in at the commit number "
                             "specified. Any changes that are not committed"
                             " will be lost forever. \nAre you sure you wish"
                             " to revert? (yes/no) ")
        if response == 'yes' or response == 'y':
            print "Reverting....please wait..."
            hf.revert(sys.argv[2], sys.argv[3])
            print "Revert finished. You are now at the state of " \
                  "commit number %s in branch %s" % (sys.argv[3], sys.argv[2])
        elif response == 'no' or response == 'n':
            print "Cancelling revert"
        else:
            print "Invalid response, cancelling"
    else:
        print "Commit number is invalid, aborting revert."
        print "Please form revert commands by typing" \
              " '$jet revert <branch_name> <commit_number>' \n" \
              "Remember the default branch is called 'master'"


def run():
    revert()



from jet_files import helper_functions as hf


def help_text():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    print "For help, please visit www.jetvc.co.uk/documentation/"


def run():
    help_text()



from shutil import copyfile
from jet_files import helper_functions as hf
import os
from os import walk


def init():
    # Can't initialize in a folder already controlled by Jet.
    if hf.already_initialized():
        print "Already a repo initialized"
    else:
        # This is the directory all Jets information will be stored under
        os.mkdir('.jet')
        f = []  # Full filenames
        filenames_list = []  # Names of the files
        # Goes through all files in the current directory and below.
        for (dirpath, dirnames, filenames) in walk(os.getcwd()):
            for filename in filenames:
                # Checks they're not in the jet ignore file
                if hf.filter_one_file_by_ignore(filename):
                    filenames_list.append(filename)
                    f.append(os.path.join(dirpath, filename))
        # Store a copy of the files being source controlled
        with open('.jet/latest_saved_files', 'w') as file_:
            for file_to_add in f:
                # Storing full filename
                file_.write(file_to_add + "\n")
                # Storing hash of the file contents
                file_.write(hf.checksum_md5(file_to_add) + "\n")
        # Making an initial commit 
        os.mkdir('.jet/0/')
        # Storing the files being saved in the commit
        with open('.jet/0/file_log.txt', 'w') as file_:
            for file_to_add in f:
                file_.write(file_to_add + "\n")

        # Going through each file storing their initial contents
        count = 0
        for file_to_add in f:
            # A folder for each file, to store filename
            # and contents separately.
            folder = os.path.join(hf.get_jet_directory() +
                                  '/.jet/%s/%s' % (0, count))
            os.mkdir(folder)
            # Storing filename..
            filename = os.path.join(hf.get_jet_directory() +
                                    '/.jet/%s/%s/filename.txt'
                                    % (0, count))
            with open(filename, 'w') as myFile:
                    myFile.write(file_to_add)
            filename = filenames_list[count]
            # Copying the contents over, to enable diffs to work
            copyfile(file_to_add, '.jet/0/%s/%s' % (count, filename))
            count += 1
            
        # Storing the current branch that's being worked on
        filename = '.jet/branch'
        with open(filename, 'w') as file_:
            file_.write("master")
        # Sets the current commit to 0
        filename = '.jet/current_commit'
        with open(filename, 'w') as file_:
            file_.write("0")
        # Init is all done, inform user.
        print hf.BColors.GREEN + \
            "Initializing Jet repository in %s" % os.getcwd() + hf.BColors.ENDC


def run():
    init()



from jet_files import helper_functions as hf
import sys
import os


def list_commits():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    try:
        commit_number = sys.argv[2]
        if commit_number == "0":
            print "Initial commit"
            return
    except IndexError:
        folder = hf.get_branch_location()
        commits = hf.get_immediate_subdirectories(folder)
        print "List of all commits. Type '$jet list <commit_number>' " \
              "to see more information"
        commits.reverse()
        for commit_num in commits:
            print "Commit number: %s" % commit_num
        return
    folder = hf.get_branch_location()
    commits = hf.get_immediate_subdirectories(folder)
    found = False
    for commit in commits:
        if commit == commit_number:
            found = True
            filename = os.path.join(hf.get_branch_location() +
                                    '/%s/file_log.txt' % commit_number)
            with open(filename, 'r') as file_:
                lines = file_.read().splitlines()
            try:
                line_number = sys.argv[3]
                try:
                    line = lines[int(line_number)]
                    try:
                        filename = os.path.join(hf.get_branch_location()
                                                + '/%s/%s/changes.txt' %
                                                (commit_number, line_number))
                        with open(filename, 'r') as file_:
                            to_print = file_.read().splitlines()
                        print "Changes to file %s" % line
                        for line_to_print in to_print:
                            print "    %s" % line_to_print
                    except IOError:
                        print "That line number was not " \
                              "an edited file, therefore there is no more" \
                              " information on it"
                except (IndexError, TypeError):
                    print "Incorrect line number, type " \
                          "'$jet list %s' to see the possible" \
                          " lines numbers." % commit_number
                return
            except IndexError:
                pass
            print "Changelog for commit number %s" % commit_number
            print ""
            count = 0
            for line in lines:
                print "Line number %s:    %s" % (count, line)
                count += 1
            print ""
            print "Type '$jet list %s <line_number>'" \
                  " to see more information about that change" % commit_number

    if not found:
        print "Incorrect commit number, type '$jet list' to see all commits"


def run():
    list_commits()


import os
from jet_files import helper_functions as hf


# At the moment, all files are added to the changeset, issue raised to change
def add(verbose):
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return

    filename = os.path.join(hf.get_branch_location() + 'changeset.txt')
    current_files = hf.get_current_files(None)
    stored_files_and_hashes = hf.get_stored_files_and_hashes()
    stored_files = hf.get_stored_files(stored_files_and_hashes)
    with open(filename, 'w') as file_:
        for file_to_add in hf.get_new_files(current_files, stored_files):
            file_.write("+" + file_to_add + "\n")
        for file_to_add in hf.get_deleted_files(current_files, stored_files):
            file_.write("-" + file_to_add + "\n")
        for file_to_add in hf.get_changed_files(current_files,
                                                stored_files_and_hashes):
            file_.write("~" + file_to_add + "\n")

    if verbose:
        print "Added to changeset"


def run():
    add(verbose=True)



import os
import sys
from jet_files import helper_functions as hf
import requests
import json

#DOMAIN = 'http://0.0.0.0:8000/'
DOMAIN = 'http://www.jetvc.co.uk/'


def push():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    jet_directory = hf.get_jet_directory()
    branch = hf.get_branch()
    if not hf.is_setup():
        print "You must setup before pushing! To do this type:" \
              " $jet setup"
        return
    hook = hf.get_push_hook()
    if hook:
        result = hf.run_hook(hook)
        if result:
            print "Hook passed."
        else:
            print "Hook Failed. Not pushing"
            return
    branch, last_update = hf.get_last_update(branch)
    current_commit = hf.get_commit()
    if int(last_update) == int(current_commit):
        print "No changes to push"
        return

    print "Connecting...."
    url = "%scurrent_file_list/%s/%s/" % (DOMAIN,
                                          hf.get_repo_id(),
                                          branch)
    try:
        response = requests.get(url)
        content = json.loads(response.content)
    except Exception, e:
        print "Failed to connect to Jets servers."
        print "Error - %s" % e
        return
    # Change to only commited files
    current_files = hf.get_file_list_at(branch, hf.get_commit())
    # Send commit POST
    print "Creating commit on server..."
    url = "%screate_commit/" % DOMAIN
    if sys.argv[2] == "-m" and len(sys.argv) == 4:
        message = sys.argv[3]
    else:
        message = "Pushed from local servers"
    data = {
        'message': message,
        'user_id': hf.get_user_id(),
        'branch_name': branch,
        'repo_id': hf.get_repo_id(),
    }
    response = requests.post(url, data=data)
    commit_id = response.headers['commit_id']
    for _file in content['files']:
        filename = jet_directory + '/' + _file['filename']
        try:
            current_files.remove(filename)
        except ValueError:
            continue
        try:
            current_hash = hf.checksum_md5(filename)
        except IOError:
            current_hash = "Not a file"
        if current_hash == _file['hash']:
            # File is unchanged
            continue
        with open(filename, 'r') as myFile:
            contents = myFile.read()
        print "Uploading file %s..." % filename
        stripped_filename = filename[len(jet_directory):]
        if stripped_filename.startswith('/'):
            stripped_filename = stripped_filename[1:]
        url = "%supdate_file/" % DOMAIN
        data = {
            'filename': stripped_filename,
            'api_key': hf.get_user_id(),
            'branch_name': branch,
            'repo_id': hf.get_repo_id(),
            'contents': contents,
            'commit_id': commit_id,
        }
        requests.post(url, data=data)

    for new_file in current_files:
        filename = new_file
        with open(filename, 'r') as myFile:
            contents = myFile.read()
        print "Uploading file %s..." % hf.relative(filename, os.getcwd())
        stripped_filename = filename[len(jet_directory):]
        if stripped_filename.startswith('/'):
            stripped_filename = stripped_filename[1:]
        url = "%supdate_file/" % DOMAIN
        data = {
            'filename': stripped_filename,
            'branch_name': branch,
            'api_key': hf.get_user_id(),
            'repo_id': hf.get_repo_id(),
            'contents': contents,
            'commit_id': commit_id,
        }
        requests.post(url, data=data)

    hf.add_update(branch, hf.get_commit())
    last_server_pull = hf.get_last_server_pull(branch)
    hf.save_last_pull(branch, last_server_pull + 1)
    print hf.BColors.GREEN + "Pushed" + hf.BColors.ENDC


def run():
    push()


import os
import sys
from jet_files import helper_functions as hf


def diff():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if len(sys.argv) != 5:
        print "Please form diff commands " \
              "'$jet diff <filename> <branch> <commit_number>'"
    if os.path.exists(sys.argv[2]):
        full_file_name = os.path.join(os.getcwd() + '/' + sys.argv[2])
        old_file = hf.get_file_at(sys.argv[3], sys.argv[4], full_file_name)
        if old_file is None:
            print "File could not be found at the point specified"
            return
        with open(full_file_name, 'r') as file_:
            current_file = file_.read().splitlines()
        difference = hf.diff(old_file, current_file)
        print difference
    else:
        print "Invalid filename."


def run():
    diff()


import os
import sys
from jet_files import helper_functions as hf


def resolve():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if not len(sys.argv) == 3:
        print "Please form resolve statements '$jet resolve <filename>'"
    file_to_resolve = os.path.join(os.getcwd() + '/' + sys.argv[2])
    result = hf.resolve_conflict(file_to_resolve)
    if result == -1:
        print "That file does not need resolving"
        return
    print "File has been resolved"
    if result > 0:
        print "You have %s conflicts left to resolve" % result
    else:
        print "All files resolved, remember to commit!"


def list_conflicts():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    conflicts = hf.get_conflicts()
    if conflicts:
        print "You have the following merge conflicts to resolve:"
        for conflict in conflicts:
            print "    %s" % conflict
    else:
        print "No merge conflicts are present!"


import os
import shutil
import helper_functions as hf


def stash():
    jet_directory = hf.get_jet_directory()
    stash_path = os.path.join(jet_directory + '/.jet/stash/')
    if os.path.exists(stash_path):
        response = raw_input("Are you sure you wish to overwrite your"
                             " previous stash? (y/n) ")
        if not response == "y" or response == "yes":
            print "Cancelling...."
            return
        shutil.rmtree(stash_path)
    os.mkdir(stash_path)
    f = []  # Full filenames
    filenames_list = []  # Names of the files
    # Goes through all files in the current directory and below.
    for (dirpath, dirnames, filenames) in os.walk(jet_directory):
        for filename in filenames:
            # Checks they're not in the jet ignore file
            if hf.filter_one_file_by_ignore(filename):
                if not dirpath\
                        .startswith(os.path.join(jet_directory + '/.jet')):
                    filenames_list.append(filename)
                    f.append(os.path.join(dirpath, filename))
    count = 0
    for file_to_add in f:
        # A folder for each file, to store filename and contents separately.
        folder = os.path.join(stash_path + '/%s' % count)
        os.mkdir(folder)
        # Storing filename..
        filename = os.path.join(stash_path + '/%s/filename.txt' % count)
        with open(filename, 'w') as myFile:
                myFile.write(file_to_add)
        filename = filenames_list[count]
        # Copying the contents over, to enable diffs to work
        new_filename = os.path.join(stash_path + '/%s/%s' % (count, filename))
        shutil.copyfile(file_to_add, new_filename)
        count += 1
    print "Successfully stashed the code"


def unstash():
    jet_directory = hf.get_jet_directory()
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    stash_path = os.path.join(jet_directory + '/.jet/stash/')
    if not os.path.exists(stash_path):
        print "Cannot unstash as there is no stashed content"
        return
    print "Are you sure you wish to restore the code that is stashed?" \
          " Any un-committed changes will be lost."
    response = raw_input("This action is irreversible. (y/n) ")
    if not response == "y" or response == "yes":
        print "Cancelling..."
        return

    current_files = hf.get_current_files(None)
    files = os.listdir(stash_path)
    for _file in files:
        stored_filename_filename = os.path.join(stash_path
                                                + '/%s/filename.txt' % _file)
        with open(stored_filename_filename, 'r') as myFile:
            filename = myFile.read()
        try:
            current_files.remove(filename)
            new = False
        except ValueError:
            new = True
        try:
            current_hash = hf.checksum_md5(filename)
        except IOError:
            current_hash = "Not a file"
        head, name = os.path.split(filename)
        stored_contents_filename = os.path.join(stash_path +
                                                '/%s/%s' % (_file, name))
        with open(stored_contents_filename, 'r') as myFile:
            stored_contents = myFile.read()
        stored_contents_hash = hf.checksum_md5(stored_contents_filename)
        if current_hash == stored_contents_hash:
            # File is unchanged
            continue
        # File is either new or edited
        if new:
            hf.make_directories(filename, clone=False)
        print "Updating %s" % hf.relative(filename, os.getcwd())
        with open(filename, 'w') as new_file:
            new_file.write(stored_contents)

    for file_to_delete in current_files:
        os.remove(file_to_delete)

    print "Loaded in the code from the stash"


def run():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return

    stash()


#Purpose of this file is to test the local version of the version control
import os
import shutil
from jet_files import helper_functions, init, add, commit_changeset
from jet_files import branch as b

RESULTS = []
EXPECTED_RESULT = "No changes found"
cwd = os.getcwd()
base_tests_path = os.path.join(os.path.dirname(os.path.realpath(__file__))
                               + '/../')
directory = os.path.join(cwd + '/tests_test_directory/')
file1 = os.path.join(directory + 'one.py')
file2 = os.path.join(directory + 'one/two.py')
file3 = os.path.join(directory + 'one/one.py')
file4 = os.path.join(directory + 'three.py')
file5 = os.path.join(directory + 'four.py')
file6 = os.path.join(directory + 'five/five.py')
file7 = os.path.join(directory + 'five/six/seven/eight/six.py')


def test_same_files():
    test_list = ['1',
                 '2',
                 '3',
                 '4',
                 '5']
    for test in test_list:
        diff = helper_functions.diff(os.path.join(base_tests_path
                                     + 'tests/diff/same/%s/before.txt' % test),
                                     os.path.join(base_tests_path
                                     + "tests/diff/same/%s/after.txt" % test))

        if diff == EXPECTED_RESULT:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Same Test '%s' Failed" % test)
            RESULTS.append("Expected %s" % EXPECTED_RESULT)
            RESULTS.append("Received:\n %s" % diff)


def test_different_files():
    number_of_tests = 20
    for test in range(0, number_of_tests + 1):
        diff = helper_functions.diff(os.path.join(base_tests_path
                                     + 'tests/diff/different/%s/before.txt'
                                     % test),
                                     os.path.join(base_tests_path
                                     + "tests/diff/different/%s/after.txt"
                                     % test))

        answer = helper_functions.reform_file(os.path.join(base_tests_path
                                              + 'tests/diff/different/'
                                              '%s/before.txt' % test),
                                              diff.splitlines())

        difference = helper_functions.diff(os.path.join(base_tests_path
                                           + 'tests/diff/different'
                                             '/%s/after.txt'
                                           % test),
                                           answer)

        if difference == EXPECTED_RESULT:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Different Test '%s' Failed" % test)


def test_diff_algorithm():
    print "Testing diff algorithm"
    try:
        test_same_files()
    except Exception, e:
        print e
        RESULTS.append('FAILED DIFF')
    try:
        test_different_files()
    except Exception, e:
        print e
        RESULTS.append('FAILED DIFF')


def test_merges():
    number_of_tests = 15
    for i in range(1, number_of_tests + 1):
        parent_filename = os.path.join(base_tests_path
                                       + 'tests/merge/%s/parent.txt' % i)
        with open(parent_filename, 'r') as myFile:
            parent = myFile.read().splitlines()
        file1_filename = os.path.join(base_tests_path
                                      + 'tests/merge/%s/file1.txt' % i)
        with open(file1_filename, 'r') as myFile:
            fileone = myFile.read().splitlines()
        file2_filename = os.path.join(base_tests_path
                                      + 'tests/merge/%s/file2.txt' % i)
        with open(file2_filename, 'r') as myFile:
            filetwo = myFile.read().splitlines()
        result_filename = os.path.join(base_tests_path
                                       + 'tests/merge/'
                                         '%s/expected_result.txt' % i)
        with open(result_filename, 'r') as myFile:
            expected_result = myFile.read().splitlines()
        result = helper_functions.fix_file("irrelevant",
                                           parent, fileone, filetwo, test=True)
        split_result = result[0].splitlines()
        if helper_functions.diff(split_result, expected_result) \
                == EXPECTED_RESULT \
                or helper_functions.diff(result, expected_result)\
                == EXPECTED_RESULT:
            RESULTS.append('Passed')
        else:
            RESULTS.append("Merging Test '%s' Failed" % i)


def test_merging():
    print "Testing merging algorithm"
    try:
        test_merges()
    except Exception, e:
        RESULTS.append('FAILED MERGING')
        print e


def test_get_jet_directory():
    expected_jet_directory = os.path.join(cwd + '/tests_test_directory')
    jet_directory = helper_functions.get_jet_directory()
    if jet_directory == expected_jet_directory:
        RESULTS.append("Passed")
    else:
        RESULTS.append("Failed get jet directory. Received %s, should be %s"
                       % (jet_directory, expected_jet_directory))


def setup():
    os.mkdir(directory)

    with open(file1, 'w') as myFile:
        myFile.write(file1)

    os.mkdir(os.path.join(directory + 'one/'))
    with open(file2, 'w') as myFile:
        myFile.write(file2)

    with open(file3, 'w') as myFile:
        myFile.write(file3)

    with open(file4, 'w') as myFile:
        myFile.write(file4)

    with open(file5, 'w') as myFile:
        myFile.write(file5)

    os.mkdir(os.path.join(directory + 'five/'))
    os.mkdir(os.path.join(directory + 'five/six/'))
    os.mkdir(os.path.join(directory + 'five/six/seven/'))
    os.mkdir(os.path.join(directory + 'five/six/seven/eight/'))

    with open(file6, 'w') as myFile:
        myFile.write(file6)

    with open(file7, 'w') as myFile:
        myFile.write(file7)

    os.chdir(directory)
    init.init()


def clear_up():
    shutil.rmtree(directory)
    os.chdir(cwd)


def test_current_files():
    expected_files = [file1, file2, file3, file4, file5, file6, file7]
    current_files = helper_functions.get_current_files(None)
    for f in expected_files:
        if f not in current_files:
            RESULTS.append("Failed current files")
        else:
            RESULTS.append("Passed")
    if not len(expected_files) == len(current_files):
        RESULTS.append("Failed current files length check")
    else:
        RESULTS.append("Passed")


def test_get_new_commit_number(expected_result):
    new_commit = helper_functions.get_new_commit_number()
    if not expected_result == new_commit:
        RESULTS.append("Failed get new commit")
    else:
        RESULTS.append("Passed")


def test_stored_files():
    expected_files = [file1, file2, file3, file4, file5, file6, file7]
    stored_files = helper_functions.get_stored_files(None)
    for f in expected_files:
        if f not in stored_files:
            RESULTS.append("Failed stored files")
        else:
            RESULTS.append("Passed")
    if not len(expected_files) == len(stored_files):
        RESULTS.append("Failed stored files length check")
    else:
        RESULTS.append("Passed")


def test_get_stored_hash():
    files = [file1, file2, file3, file4, file5, file6, file7]
    for f in files:
        file1hash = helper_functions.checksum_md5(f)
        stored_hash = helper_functions.get_stored_hash(f, None)
        if not file1hash == stored_hash:
            RESULTS.append("Failed stored hash")
        else:
            RESULTS.append("Passed")


def test_get_new_files_in_changeset(expected_result):
    new_files = helper_functions.get_new_files_in_changeset(None)
    if not len(new_files) == expected_result:
        RESULTS.append("Failed new files in changeset")
    else:
        RESULTS.append("Passed")


def test_get_deleted_files_in_changeset(expected_result):
    deleted_files = helper_functions.get_deleted_files_in_changeset(None)
    if not len(deleted_files) == expected_result:
        RESULTS.append("Failed deleted files in changeset")
    else:
        RESULTS.append("Passed")


def test_get_changed_files_in_changeset(expected_result):
    changed_files = helper_functions.get_changed_files_in_changeset(None)
    if not len(changed_files) == expected_result:
        RESULTS.append("Failed changed files in changeset")
    else:
        RESULTS.append("Passed")


def test_get_deleted_files(expected_result):
    deleted_files = helper_functions.get_deleted_files(None, None)
    if len(deleted_files) == expected_result:
        RESULTS.append('Passed')
    else:
        RESULTS.append('Failed deleted files test')


def test_get_changed_files(changed_file):
    changed_files = helper_functions.get_changed_files(None, None)
    if changed_file == 0:
        if len(changed_files) == 0:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Failed amount of changed files")
    else:
        if len(changed_files) == 1:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Failed amount of changed files")
        if changed_files[0] == changed_file:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Failed which file was changed")


def test_get_file_change_number(filename, branch):
    result = helper_functions.get_file_change_number(branch, 1, filename)
    expected_result = 0

    if result == expected_result:
        RESULTS.append('Passed')
    else:
        RESULTS.append('Incorrect file change number')


def test_get_last_complete_file(branch):
    result, commit = helper_functions.get_last_complete_file(branch, file7)
    expected_result, commit_number = [file7], 0

    if result == expected_result:
        RESULTS.append('Passed')
    else:
        RESULTS.append('Failed last complete file. Received %s' % result)

    if commit == commit_number:
        RESULTS.append('Passed')
    else:
        RESULTS.append('Failed last complete file. Received %s' % commit)


def test_get_file_at(commit, expected_result, branch):
    result = helper_functions.get_file_at(branch, commit, file7)
    if result == expected_result:
        RESULTS.append('Passed')
    else:
        RESULTS.append("Failed 'get file at' test")


def test_get_highest_commit(expected_result, branch):
    result = helper_functions.get_highest_commit(branch)
    if result == expected_result:
        RESULTS.append('Passed')
    else:
        RESULTS.append('Get highest commit failed. Got %s, expected %s'
                       % (result, expected_result))


def test_hashing_algorithm_is_unique():
    # Used to debug, not really needed anymore...
    tests = [
        ["", "", "", "", "", ""],
        ["", "", "", "", ""],
        [""],
        []
    ]
    checksums = []
    for i in range(0, len(tests)):
        filename = os.path.join(directory + 'test%s' % i)
        with open(filename, 'w') as myFile:
            for line in tests[i]:
                myFile.write("%s\n" % line)
        checksums.append(helper_functions.checksum_md5(filename))
        os.remove(filename)

    if len(checksums) > len(set(checksums)):
        RESULTS.append("Checksum has a bug")
    else:
        RESULTS.append("Passed")


def test_filtering_files_by_jet_ignore():
    test_filenames = [
        'one.py',
        'one.py~',
        'one.pyc',
        os.getcwd() + 'nope/one.py',
    ]
    expected_result = [
        'one.py'
    ]
    filters = [
        '*~',
        '*.pyc',
        'nope/',
    ]
    with open(os.path.join(directory + '/.jet_ignore'), 'w') as myFile:
        for line in filters:
            myFile.write("%s\n" % line)
    result = helper_functions.filter_files_by_ignore(test_filenames)
    for f in expected_result:
        if f in result:
            RESULTS.append('Passed')
        else:
            RESULTS.append('Failed jet ignore')
    if len(expected_result) == len(result):
        RESULTS.append('Passed')
    else:
        RESULTS.append('Failed length test for jet ignore')


def test_relative():
    currentwd = '/home/connor/development/project/jet/tests/test_directory/'
    filename = [
        '/home/connor/development/project/jet/tests/test_directory/one',
        '/home/connor/development/project/jet/tests/other/one',
        '/home/connor/development/project/jet/other/other2/one',
        '/home/connor/development/project/jet/tests/test_directory/folder/one',
        '/home/connor/development/project/jet/tests/one',
    ]
    expected_answer = [
        'one',
        '../other/one',
        '../../other/other2/one',
        'folder/one',
        '../one',
    ]
    for i in range(0, len(filename)):
        if helper_functions.relative(filename[i],
                                     currentwd) == expected_answer[i]:
            RESULTS.append("Passed")
        else:
            RESULTS.append("Failed relative. Got '%s', expected '%s'"
                           % (helper_functions.relative(filename[i],
                                                        currentwd),
                              expected_answer[i]))


def test_branch(branch):
    test_get_jet_directory()
    test_current_files()
    test_stored_files()
    test_get_new_commit_number(1)
    test_get_stored_hash()
    test_get_new_files_in_changeset(0)
    test_get_deleted_files_in_changeset(0)
    test_get_changed_files_in_changeset(0)
    test_get_deleted_files(0)
    test_get_changed_files(0)
    test_get_last_complete_file(branch)
    test_get_file_at('0', expected_result=[file7], branch=branch)

    os.remove(file7)
    test_get_deleted_files(1)

    new_contents1 = ["", "", "", "", file7, "", "", file7, "", ""]
    with open(file7, 'w') as myFile:
        for line in new_contents1:
            myFile.write("%s\n" % line)
    test_get_changed_files(file7)
    test_get_highest_commit("0", branch)

    add.add(verbose=False)
    test_get_changed_files_in_changeset(1)
    commit_changeset.commit("This is a test message", verbose=False)
    test_get_highest_commit("1", branch)
    test_get_changed_files(0)
    test_get_file_change_number(file7, branch)
    test_get_last_complete_file(branch)
    test_get_file_at('1', expected_result=new_contents1, branch=branch)

    os.remove(file6)
    random_new_file = os.path.join(directory + 'random')
    with open(random_new_file, 'w') as myFile:
        myFile.write("lol")
    helper_functions.revert(branch, '0')
    test_get_file_at('0', [file7], branch)
    test_stored_files()

    new_contents2 = ["", "", "", "", "", ""]
    with open(file7, 'w') as myFile:
        for line in new_contents2:
            myFile.write("%s\n" % line)

    add.add(verbose=False)
    commit_changeset.commit("This is a test message", verbose=False)

    test_get_file_at("2", expected_result=new_contents2, branch=branch)
    test_get_file_at('1', expected_result=new_contents1, branch=branch)
    test_get_file_at('0', expected_result=[file7], branch=branch)

    test_get_changed_files(0)
    test_get_deleted_files(0)
    test_get_new_commit_number(3)
    test_current_files()


def test_common_functions():
    print "Testing common functions"
    setup()
    try:
        test_branch('master')

        clear_up()
        setup()
        b.branch('test_branch')
        test_branch('test_branch')

        clear_up()
        setup()
        b.branch('test_branch')
        b.branch('second_test_branch')
        test_branch('second_test_branch')

        test_filtering_files_by_jet_ignore()
        test_relative()
        test_hashing_algorithm_is_unique()
    except Exception, e:
        RESULTS.append('FAILED')
        print e
    clear_up()


def test_dependencies():
    try:
        import requests
        RESULTS.append('Passed')
    except Exception, e:
        RESULTS.append('Failed importing requests')
        print e


def run():
    print "Beginning tests..."
    test_dependencies()

    test_diff_algorithm()
    test_merging()
    test_common_functions()

    print "Results:"
    number_of_tests = len(RESULTS)
    passed = 0
    for result in RESULTS:
        if result == 'Passed':
            passed += 1
            print helper_functions.BColors.GREEN + \
                "    Passed" + helper_functions.BColors.ENDC
        else:
            print helper_functions.BColors.RED +\
                "    %s" % result + helper_functions.BColors.ENDC

    print "Passed %s out of %s tests" % (passed, number_of_tests)



import os
import sys
from jet_files import helper_functions as hf
from jet_files import commit_changeset, add


def merge():
    # merging is based on
    # http://git-scm.com/book/en/v2/Git-Branching-Basic-Branching-and-Merging
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if not len(sys.argv) == 3:
        print "Please form merge commands '$jet merge" \
              "<branch>' - where <branch> is the branch to merge into" \
              "the current one. \nThe merge will make a new commit" \
              " in the current branch"
        return
    filename = os.path.join(hf.get_jet_directory() + '/.jet/changeset.txt')
    changed_files = False
    if os.path.isfile(filename):
        changed_files = True
    if len(hf.get_changed_files(None)) > 0:
        changed_files = True
    if changed_files:
        print "You can't merge until you commit...."
        return
    branch = sys.argv[2]
    branches_path = os.path.join(hf.get_jet_directory() + '/.jet/branches/')
    if os.path.exists(branches_path):
        if not os.path.exists(os.path.join(branches_path + sys.argv[2])):
            print "Invalid branch name, please try again."
            return

    hf.merge(branch)
    add.add(verbose=False)
    commit_changeset.commit("Merged branch %s into branch %s."
                            % (branch, hf.get_branch()), verbose=False)
    commit_number_used = hf.get_new_commit_number() - 1
    print "Merged branch %s into branch current branch %s." \
          "\nCommitted merge with commit number %s." % (hf.get_branch(),
                                                        branch,
                                                        commit_number_used)


def run():
    merge()


from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file

setup(
    name='jet',
            
    version='0.0.8',
            
    description='A simple to user, yet powerful version control system',
            
    long_description="See www.jetvc.co.uk",
            
    url='https://github.com/entangly/jet/',
            
    author='Connor Swain',
            
    author_email='connor@bradingsoftware.co.uk',
            
    license='MIT',
            
    classifiers=[
                'Development Status :: 3 - Alpha',
        
                'Intended Audience :: Developers',
        
                'Topic :: Software Development :: Version Control',
                
                'License :: OSI Approved :: MIT License',

                'Programming Language :: Python :: 2.7',
    ],
            
    keywords='version control jet',
            
    packages=find_packages(),

    entry_points={'console_scripts': ['jet = jet_files.jet:run'],},

            
    install_requires=['requests',],
)



import json
import os
import sys
from jet_files import helper_functions as hf
from jet_files import init, commit_changeset, add
import requests


#DOMAIN = 'http://0.0.0.0:8000/'
DOMAIN = 'http://www.jetvc.co.uk/'


def force_pull():
    branch = hf.get_branch()
    jet_directory = hf.get_jet_directory()
    url = "%scurrent_file_list/%s/%s/" % (DOMAIN,
                                          hf.get_repo_id(),
                                          branch)
    response = requests.get(url)
    content = json.loads(response.content)
    current_files = hf.get_current_files(None)
    for _file in content['files']:
        filename = jet_directory + '/' + _file['filename']
        try:
            current_files.remove(filename)
            new = False
        except ValueError:
            new = True
        try:
            current_hash = hf.checksum_md5(filename)
        except IOError:
            current_hash = "Not a file"
        if current_hash == _file['hash']:
            # File is unchanged
            continue
        # File is either new or edited
        print "Updating file %s..." % hf.relative(filename, os.getcwd())
        url = "%sapi/v1/file/%s/?api_key=%s" % (DOMAIN,
                                                _file['file_id'],
                                                hf.get_user_id())
        response = requests.get(url)
        content = json.loads(response.content)
        new_contents = content['contents']
        if new:
            hf.make_directories(filename, clone=False)
        with open(filename, 'w') as new_file:
            new_file.write(new_contents)

    for file_to_delete in current_files:
        os.remove(file_to_delete)


def clone():
    if hf.already_initialized():
        print "Already a repo here, can't clone into this directory."
        return
    if not len(sys.argv) == 4:
        print "Please form clone commands '$jet clone <repo_id> <branch_name>'"
        return
    branch = sys.argv[3]
    repo_id = sys.argv[2]
    directory = os.getcwd()
    if not os.listdir(directory) == []:
        print "Can only clone into an empty directory, sorry."
        return
    print "Connecting...."
    url = "%scurrent_file_list/%s/%s/" % (DOMAIN,
                                          repo_id,
                                          branch)
    try:
        response = requests.get(url)
        content = json.loads(response.content)
    except Exception, e:
        print "Failed to connect to Jets servers."
        print "Error - %s" % e
        return
    if not response.status_code == 200:
        print "There was an error with the repo id or branch name " \
              "- please try again!!!"
        return
    for _file in content['files']:
        filename = directory + '/' + _file['filename']
        print "Adding file %s..." % hf.relative(filename, directory)
        url = "%sapi/v1/file/%s/?api_key=%s" % (DOMAIN,
                                                _file['file_id'],
                                                hf.get_user_id())
        response = requests.get(url)
        content = json.loads(response.content)
        new_contents = content['contents']
        hf.make_directories(filename, clone=True)
        with open(filename, 'w') as new_file:
            new_file.write(new_contents)
    init.run()
    print hf.BColors.GREEN + "Successfully cloned the repo." + hf.BColors.ENDC


def pull():
    branch = hf.get_branch()
    jet_directory = hf.get_jet_directory()
    url = "%scurrent_file_list/%s/%s/" % (DOMAIN,
                                          hf.get_repo_id(),
                                          branch)
    response = requests.get(url)
    content = json.loads(response.content)
    current_files = hf.get_current_files(None)
    server_files = [jet_directory + '/' + _file['filename']
                    for _file in content['files']]
    server_ids = [_file['file_id'] for _file in content['files']]
    parent_branch, parent_commit_number = hf.get_last_update(branch)
    parent_files = hf.get_file_list_at(parent_branch, parent_commit_number)

    files_to_merge = []
    files_to_ask_about = []

    for file_ in current_files:
        if file_ in server_files:
            files_to_merge.append(file_)
        else:
            files_to_ask_about.append(file_)
    files_to_ask_about += [x for x in server_files if x not in current_files]

    for f in files_to_ask_about:
        answer = hf.ask(f)
        if not answer:
            os.remove(f)
        else:
            if f not in current_files:
                print "Downloading file %s..." % hf.relative(f, os.getcwd())
                url = "%sapi/v1/file/%s/?api_key=%s" \
                      % (DOMAIN,
                         server_ids[server_files.index(f)],
                         hf.get_user_id())
                response = requests.get(url)
                content = json.loads(response.content)
                new_contents = content['contents']
                with open(f, 'w') as myFile:
                    myFile.write(new_contents)
    print "Checking for changes / merges"
    for f in files_to_merge:
        if f in parent_files:
            parent_file = hf.get_file_at(parent_branch,
                                         parent_commit_number,
                                         f)
        else:
            parent_file = []

        with open(f, 'r') as myFile:
            local_file = myFile.read().splitlines()

        url = "%sapi/v1/file/%s/?api_key=%s"\
              % (DOMAIN,
                 server_ids[server_files.index(f)],
                 hf.get_user_id())
        response = requests.get(url)
        content = json.loads(response.content)
        new_contents = content['contents']
        server_file = new_contents.splitlines()

        if not local_file == server_file:
            try:
                new_file = hf.fix_file(f, parent_file, local_file, server_file)
            except IndexError:
                # error has happened. Apply worst case scenario.
                new_file = \
                    ['@@@@@@@@@@HEAD@@@@@@@@@@\n'] \
                    + local_file \
                    + ['\n@@@@@@@@@@SEPARATOR@@@@@@@@@@\n'] \
                    + server_file \
                    + ['\n@@@@@@@@@@END@@@@@@@@@@']
                hf.add_conflict(f)
            with open(f, 'w') as myFile:
                for line in new_file:
                    myFile.write('%s\n' % line)


def run():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    branch = hf.get_branch()
    print "Connecting...."
    url = "%shighest_commit/%s/%s/" % (DOMAIN,
                                       hf.get_repo_id(),
                                       branch)
    try:
        response = requests.get(url)
        content = json.loads(response.content)
    except Exception, e:
        print "Failed to connect to Jets servers."
        print "Error - %s" % e
        return
    server_commit = content['commit_number']
    print "Connected."
    last_server_pull = hf.get_last_server_pull(branch)
    if server_commit == last_server_pull:
        print "You are already upto date."
        return
    try:
        if sys.argv[2] == '-f':
            force_pull()
        else:
            print "Invalid argument."
            return
    except IndexError:
        pull()
    add.add(verbose=False)
    commit_changeset.commit("Merged branch %s with servers changes."
                            % branch, verbose=False)
    hf.save_last_pull(branch, server_commit)
    print "Committed merges"
    print hf.BColors.GREEN + "Pulled" + hf.BColors.ENDC


from shutil import copyfile
import helper_functions as hf
import sys
import os


def commit_changeset():
    if not hf.already_initialized():
        print "Please init a jet repo before calling other commands"
        return
    if hf.is_conflicts():
        print "You can't commit with outstanding conflicts..."
        return
    # Login required in order to put a name with the commit
    if not hf.logged_in():
        print "You must login before commiting! To do this type:" \
              " $jet login <username>"
        return

    hook = hf.get_commit_hook()
    if hook:
        result = hf.run_hook(hook)
        if result:
            print "Hook passed."
        else:
            print "Hook Failed. Not commiting"
            return
    # Must have the -m and all arguments present
    if len(sys.argv) != 4 or sys.argv[2] != "-m":
        print "Commit commands need to be formed by typing:" \
              " $jet commit -m \"Your message here\""
    else:
        commit(sys.argv[3], verbose=True)


def commit(message, verbose):
    filename = os.path.join(hf.get_branch_location() + 'changeset.txt')
    # Checks to see if there are any files in the changeset at all...
    if os.path.isfile(filename):
        files_in_changeset = hf.get_files_in_changeset(None)
        new_files_in_changeset\
            = hf.get_new_files_in_changeset(files_in_changeset)
        deleted_files_in_changeset\
            = hf.get_deleted_files_in_changeset(files_in_changeset)
        changed_files_in_changeset\
            = hf.get_changed_files_in_changeset(files_in_changeset)
        new_commit_number = hf.get_new_commit_number()
        folder = os.path.join(hf.get_branch_location() +
                              '/%s/' % new_commit_number)
        # Making a folder to put all the details of the commit in
        os.mkdir(folder)
        filename = os.path.join(hf.get_branch_location()
                                + '/%s/file_log.txt'
                                % new_commit_number)
        # Record which files changed
        with open(filename, 'w')\
                as file_:
            for file_to_add in changed_files_in_changeset:
                file_.write("~" + file_to_add + "\n")
            for file_to_add in new_files_in_changeset:
                file_.write("+" + file_to_add + "\n")
            for file_to_add in deleted_files_in_changeset:
                file_.write("-" + file_to_add + "\n")

        # Recording commit message, and who committed it.
        filename = os.path.join(hf.get_branch_location() + '/%s/info'
                                % new_commit_number)
        with open(filename, 'w') as file_:
            file_.write(hf.get_username() + '\n')
            file_.write(message)
        # Recording which commit the HEAD is pointing to
        filename = os.path.join(hf.get_jet_directory()
                                + '/.jet/current_commit')
        with open(filename, 'w') as file_:
            file_.write(str(new_commit_number))

        # Counter to make unique folder name for each file change
        counter = 0
        for file_ in changed_files_in_changeset:
            # Making the folder
            folder = os.path.join(hf.get_branch_location()
                                  + '/%s/%s' % (new_commit_number,
                                                counter))
            os.mkdir(folder)

            # Recording the full filename 
            filename = os.path.join(hf.get_branch_location()
                                    + '/%s/%s/filename.txt'
                                    % (new_commit_number, counter))
            with open(filename, 'w') as myFile:
                myFile.write(file_)
            
            # Recording the diff from last commit
            filename = os.path.join(hf.get_branch_location()
                                    + '/%s/%s/changes.txt'
                                    % (new_commit_number, counter))
            description = hf.get_change_description(file_)
            if description:
                with open(filename, 'w') as myFile:
                    myFile.write(description)
            else:
                # If no difference, put the contents of the file in there
                copyfile(file_, filename)
            counter += 1

        # Remove files from changeset as they have been committed.
        filename = os.path.join(hf.get_branch_location()
                                + 'changeset.txt')
        os.remove(filename)

        # Section that updates the latest stored files

        # Gets all previous files
        lines = hf.get_stored_files(None)
        # Adds newly saved files
        lines.extend(new_files_in_changeset)
        to_keep = []
        for line in lines:
            # Only adds files not deleted
            if line not in deleted_files_in_changeset:
                to_keep.append(line)
        filename = os.path.join(hf.get_branch_location()
                                + 'latest_saved_files')
        # Delete old record
        os.remove(filename)
        with open(filename, 'w') as file_:
            for file_to_add in to_keep:
                file_.write(file_to_add + "\n")
                # Including new checksum of file, for status/diff checks
                file_.write(hf.checksum_md5(file_to_add) + "\n")
        if verbose:
            # Done!
            print "Commiting"
    else:
        if verbose:
            print "Please add files to commit using jet add before commiting!"


def run():
    commit_changeset()
